import datetime
import jdcal
import math

class RealTime(object):

    def __init__(self, start_datetime=datetime.datetime.utcnow(), time_delta=datetime.timedelta(seconds=1)):
        """
        """        
      
        self._start_datetime = start_datetime - time_delta
        self._current_datetime = self._start_datetime
        self._timedelta = time_delta


    def step(self):
        self._current_datetime += self._timedelta
        
        return True
    
    def datetime(self, sec_from_start = None):
        if sec_from_start is None:
            sec_from_start = self.elapsed_seconds
        return self._start_datetime + datetime.timedelta(seconds = sec_from_start)
    
    @property
    def current_datetime(self):
        return self._current_datetime
        
    @property
    def current_date_julian(self):
        return RealTime.datetime_to_julian_date(self.current_datetime)
        
    @property
    def timestep_seconds(self):
        return self._timedelta.total_seconds()
    
    @property
    def timestep_timedelta(self):
        return self._timedelta
    
    @property
    def elapsed_seconds(self):
        return (self._current_datetime - self._start_datetime).total_seconds()
        
    @classmethod
    def datetime_to_julian_date(cls, dt):
        frac_day = (dt.hour / 24.0 + dt.minute / 1440.0 + 
                    dt.second / 86400.0 + dt.microsecond / (86400.0 * 1e6))
        return sum(jdcal.gcal2jd(dt.year, dt.month, dt.day)) + frac_day
        
    @classmethod
    def julian_date_to_datetime(cls, julian_date):
        """Note: Julian date's lose precision at in tens of microseconds
        Separating MJD_0 from M_JD_J2000 maintains microsecond precision
        """
        year, month, day, fday = jdcal.jd2gcal(0, julian_date)
        seconds = fday*86400
        whole_seconds = int(seconds)
        # round to nearest millisecond, julian dates don't have precision to microseconds
        milliseconds = int((seconds-whole_seconds)*1000.0 + .5)
        seconds = whole_seconds + milliseconds/1000.0
        return datetime.datetime(year,month,day) + datetime.timedelta(seconds=seconds)
