import datetime
import jdcal
import math

class Time(object):
    earth_rotation_rate = 7.2921151466855E-5      # rad/s
    earth_angular_rate = 1.72027916940703639e-2   # rad/day
    earth_angular_accel = 5.07551419432269442e-15 # rad/day^2
    def __init__(self, start_datetime=datetime.datetime.now(),
                 stop_datetime=datetime.datetime.now() + datetime.timedelta(days=1),
                 timedelta=datetime.timedelta(seconds=60)):
        """
        """        
        self._start_datetime = start_datetime - timedelta
        self._stop_datetime = stop_datetime   
        self._current_datetime = start_datetime
        self._timedelta = timedelta
        self._gmst_at_j2k = 4.894961212840512   # radians
        self._j2k_epoch = datetime.datetime(2000,1,1,12,0,0,0)
        
# TODO: What would be the best way to store time? Julian date might not be bad way to go since functions depend on it?
#        self._sim_start_time = 0.0
#        self._sim_current_time = 0.0
#        self._sim_stop_time = (self._stop_datetime  - self._start_datetime).total_seconds()
#        self._sim_timestep = timedelta.total_seconds()

    def step(self):
        self._current_datetime += self._timedelta
        if self._current_datetime > self._stop_datetime:
            return False

        return True
    
    def datetime(self, sec_from_start = None):
        if sec_from_start is None:
            sec_from_start = self.elapsed_seconds
        return self._start_datetime + datetime.timedelta(seconds = sec_from_start)
        
    def days_since_j2k(self, sec_from_start = None):
        if sec_from_start is None:
            sec_from_start = self.elapsed_seconds
        date_time = self.datetime(sec_from_start)
        return (date_time - self._j2k_epoch).total_seconds() / 86400 # sec/day
   
    def gmst(self, sec_from_start = None):
        """ GMST = Greenwich Mean Sidereal Time
               also called Greenwich Hour Angle or theta g      """
        if sec_from_start is None:
            sec_from_start = self.elapsed_seconds
        days_j2k = self.days_since_j2k(sec_from_start)
        days, frac_days = divmod(days_j2k, 1)
        two_pi = 2 * math.pi
        rot_rate = Time.earth_angular_rate
        rot_accel = Time.earth_angular_accel
        # equation from astrostandards
        gmst = (self._gmst_at_j2k + days * rot_rate + 
            frac_days * (rot_rate + two_pi) + days_j2k**2 * rot_accel)
        return gmst % two_pi  # radians        
        
    def set_time(self, time):
        self._current_datetime = time - self._timedelta
        
    @property
    def current_datetime(self):
        return self._current_datetime
        
    @property
    def current_date_julian(self):
        return Time.datetime_to_julian_date(self.current_datetime)
        
    @property
    def timestep_seconds(self):
        return self._timedelta.total_seconds()
        
    @property
    def elapsed_seconds(self):
        return (self._current_datetime - self._start_datetime).total_seconds()
        
    @classmethod
    def datetime_to_julian_date(cls, dt):
        frac_day = (dt.hour / 24.0 + dt.minute / 1440.0 + 
                    dt.second / 86400.0 + dt.microsecond / (86400.0 * 1e6))
        return sum(jdcal.gcal2jd(dt.year, dt.month, dt.day)) + frac_day
