# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016, Brian Thorpe. All Rights Reserved.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
# Any coordinates stored in ECI needs to be rendered
# using the GPU RTE Technique described in Chapter 5 Vertex Transformation
# Precision of 3 Engine Design for Virtual Globes.  This applies mostly to
# probably the orbit trails and positions.  It might not be as much of an
# issue as originally thought
# This technique centers geometry relative to the camera
# It requiers the geometry be in ECI Positions...
from cymath.vector3 cimport Vector3
from cymath.vector4 cimport Vector4
from cymath.vector3f cimport Vector3f
from cymath.vector4f cimport Vector4f
from cymath.matrix4 cimport Matrix4
from cymath.matrix4f cimport Matrix4f
from cymath.math_structs cimport float2, float3, float4, float16


cdef struct double_single16:
    float16 hi
    float16 lo


cdef struct double_single4:
    float4 hi
    float4 lo


cdef struct double_single3:
    float3 hi
    float3 lo


cdef struct double_single2:
    float2 hi
    float2 lo


cdef struct double_single:
    float hi
    float lo


cdef class DoubleSingle:
    cdef double_single _value
    cpdef double to_double(self)
    cpdef DoubleSingle subtract(self, DoubleSingle other)
    cpdef DoubleSingle add(self, DoubleSingle other)
    cpdef DoubleSingle multiply(self, DoubleSingle other)


cdef double_single double_to_ds(double value)
cdef double ds_to_double(double_single value)
cdef double_single ds_subtract(double_single a, double_single b)
cdef double_single ds_add(double_single a, double_single b)
cdef double_single ds_multiply(double_single a, double_single b)
cpdef Vector4ds ds_transform(Matrix4f mhi, Matrix4f mlo,
                             Vector4f vhi, Vector4f vlo)
cpdef Vector4ds ds_multiply4(Vector4f mhi, Vector4f mlo, float vhi, float vlo)
cpdef Vector4ds ds_add4(Vector4ds a, Vector4ds b)
cdef double_single16 to_double_single16(Matrix4 m)
cdef double_single4 to_double_single4(Vector4 v)
cdef double_single3 to_double_single3(Vector3 v)
cpdef Vector4ds to_vector4ds(Vector4 v)
cpdef Vector3ds to_vector3ds(Vector3 v)
cpdef Matrix4ds to_matrix4ds(Matrix4 m)


cdef class Vector4ds:
    cdef double_single4 _vector


cdef class Vector3ds:
    cdef double_single3 _vector


cdef class Matrix4ds:
    cdef double_single16 _matrix
    cdef Matrix4f _hi
    cdef Matrix4f _lo
