# distutils: language = c++
# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016, Brian Thorpe. All Rights Reserved.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
from cymath.math_structs cimport double16
from cymath.vector3 cimport Vector3
from cymath.vector4 cimport Vector4
from cymath.quaternion cimport Quaternion
from cymath.matrix4f cimport Matrix4f

cpdef Matrix4 _create_from_cols(Vector4 col0, Vector4 col1, Vector4 col2, Vector4 col3)
cpdef Matrix4 _create_from_rows(Vector4 row0, Vector4 row1, Vector4 row2, Vector4 row3)
cpdef Matrix4 _create_rotation_x(double angle)
cpdef Matrix4 _create_rotation_y(double angle)
cpdef Matrix4 _create_rotation_z(double angle)
cpdef Matrix4 _create_from_axis_angle(Vector3 ax, double angle)
cpdef Matrix4 _create_from_euler_angles(Vector3 euler_angles)
cpdef Matrix4 _create_from_quaternion(Quaternion q)
cpdef Matrix4 _create_lookat(Vector3 eye, Vector3 target, Vector3 up)
cpdef Matrix4 _create_perspective_fov(double fov, double aspect, double znear, double zfar)
cpdef Matrix4 _create_perspective_offcenter(double x_min, double x_max, double y_min, double y_max, double znear, double zfar)
cpdef Matrix4 _create_orthographic(double width, double height, double zNear, double zFar)
cpdef Matrix4 _create_orthographic_offcenter(double left, double right, double bottom, double top, double zNear, double zFar)
cpdef Matrix4 _create_billboard(Vector3 object_position, Vector3 camera_position, Vector3 camera_up, Vector3 camera_forward)
cpdef Matrix4 _create_billboard_constrained(Vector3 object_position, Vector3 object_forward, Vector3 camera_position, Vector3 camera_up, Vector3 camera_forward, Vector3 rotate_axis)
cpdef Matrix4 _create_scale(Vector3 scale)
cpdef Matrix4 _create_translation(Vector3 translation)
cdef class Matrix4:
    cdef double16 _matrix
    cdef Py_ssize_t shape[2]
    cdef Py_ssize_t strides[2]
    cpdef Matrix4 transpose(Matrix4 self)
    cpdef Matrix4 add(Matrix4 self, Matrix4 other)
    cpdef Matrix4 subtract(Matrix4 self, Matrix4 other)
    cpdef Matrix4 scale(Matrix4 self, float scalar)
    cpdef Matrix4 multiply(Matrix4 self, Matrix4 other)
    cpdef Vector4 transform(Matrix4 self, Vector4 other)
    cpdef double determinant(Matrix4 self)
    cpdef Matrix4 invert(Matrix4 self)
    cpdef Matrix4f to_float(self)
    cpdef _cp_richcmp(self, Matrix4 other, int op)
    cdef double* to_ptr(self)