#matrix4.pyx
#distutils: language = c++
from cymath.matrix4 cimport Matrix4
from cymath.math_structs cimport float16
from cymath.vector3f cimport Vector3f
from cymath.vector4f cimport Vector4f
from cymath.quaternionf cimport Quaternionf


cpdef Matrix4f _create_from_cols(Vector4f col0, Vector4f col1, Vector4f col2, Vector4f col3)
cpdef Matrix4f _create_from_rows(Vector4f row0, Vector4f row1, Vector4f row2, Vector4f row3)
cpdef Matrix4f _create_rotation_x(float angle)
cpdef Matrix4f _create_rotation_y(float angle)
cpdef Matrix4f _create_rotation_z(float angle)
cpdef Matrix4f _create_from_axis_angle(Vector3f ax, float angle)
cpdef Matrix4f _create_from_yaw_pitch_roll(float yaw, float pitch, float roll) #z y x? Rotation?
cpdef Matrix4f _create_from_quaternion(Quaternionf q)
cpdef Matrix4f _create_lookat(Vector3f eye, Vector3f target, Vector3f up)
cpdef Matrix4f _create_perspective_fov(float fov, float aspect, float znear, float zfar)
cpdef Matrix4f _create_perspective_offcenter(float x_min, float x_max, float y_min, float y_max, float znear, float zfar)
cpdef Matrix4f _create_orthographic(float width, float height, float zNear, float zFar)
cpdef Matrix4f _create_orthographic_offcenter(float left, float right, float bottom, float top, float zNear, float zFar)
cpdef Matrix4f _create_billboard(Vector3f object_position, Vector3f camera_position, Vector3f camera_up, Vector3f camera_forward)
cpdef Matrix4f _create_billboard_constrained(Vector3f object_position, Vector3f object_forward, Vector3f camera_position, Vector3f camera_up, Vector3f camera_forward, Vector3f rotate_axis)
cpdef Matrix4f _create_scale(Vector3f scale)
cpdef Matrix4f _create_translation(Vector3f translation)
cpdef Matrix4f _create_from_matrix4(Matrix4 m)
cdef class Matrix4f:
    cdef float16 _matrix
    cdef Py_ssize_t shape[2]
    cdef Py_ssize_t strides[2]
    cpdef Matrix4f transpose(Matrix4f self)
    cpdef Matrix4f add(Matrix4f self, Matrix4f other)
    cpdef Matrix4f subtract(Matrix4f self, Matrix4f other)
    cpdef Matrix4f scale(Matrix4f self, float scalar)
    cpdef Matrix4f multiply(Matrix4f self, Matrix4f other)
    cpdef Vector4f transform(Matrix4f self, Vector4f other)
    
    cpdef float determinant(Matrix4f self)
    cpdef Matrix4f invert(Matrix4f self)
    cpdef float16 to_float16(self)
    cpdef _cp_richcmp(self, Matrix4f other, int op)
    cdef float* to_ptr(self)
