# distutils: language = c++
# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016, Brian Thorpe. All Rights Reserved.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
from libc.math cimport sin, cos, sqrt, acos, asin, fabs, atan2, M_PI, copysign
from cymath.math_structs cimport double4
from cymath.matrix4 cimport Matrix4
from cymath.vector3 cimport Vector3
from cymath.vector4 cimport Vector4


cdef Quaternion _create_from_euler_angles(Vector3 euler_angles)
cdef Quaternion _create_from_axis_angle(Vector3 axis, double radians)
cdef Quaternion _create_from_rotation_matrix(Matrix4 matrix)

cdef class Quaternion:
    cdef double4 _quaternion
    cpdef Quaternion add(Quaternion left, Quaternion right)
    cpdef Quaternion subtract(Quaternion left, Quaternion right)
    cpdef Quaternion multiply(Quaternion left, Quaternion right)
    cpdef Quaternion scale(Quaternion self, double scalar)
    cpdef double magnitude(Quaternion self)
    cpdef double magnitude_squared(Quaternion self)
    cpdef double dot(Quaternion self, Quaternion other)
    cpdef Quaternion inverse(Quaternion self)
    cpdef Quaternion normalize(Quaternion self)
    cpdef Quaternion slerp(Quaternion self, Quaternion other, double blend)
    cpdef Quaternion negate(Quaternion self)
    cpdef Vector4 to_axis_angle_rotation(Quaternion self)
    cpdef Matrix4 to_rotation_matrix(Quaternion self)
    cpdef Vector3 to_euler_angles(Quaternion self)
    cpdef _cp_richcmp(self, Quaternion other, int op)
    
