from cymath.math_structs cimport float4

from cymath.matrix4f cimport Matrix4f
from cymath.vector3f cimport Vector3f
from cymath.vector4f cimport Vector4f
#
cdef Quaternionf _create_from_yaw_pitch_roll(float yaw, float pitch, float roll)
cdef Quaternionf _create_from_axis_angle(Vector3f axis, float radians)
cdef Quaternionf _create_from_rotation_matrix(Matrix4f matrix)

cdef class Quaternionf:
    cdef float4 _quaternion
    
    cpdef Quaternionf add(Quaternionf left, Quaternionf right)
    cpdef Quaternionf subtract(Quaternionf left, Quaternionf right)
    cpdef Quaternionf multiply(Quaternionf left, Quaternionf right)
    cpdef Quaternionf multiply_scalar(Quaternionf self, float scalar)
    cpdef Quaternionf divide(Quaternionf left, Quaternionf right)
    cpdef Quaternionf divide_scalar(Quaternionf self, float scalar)    
    cpdef float magnitude(Quaternionf self)
    cpdef float magnitude_squared(Quaternionf self)
    cpdef float dot(Quaternionf self, Quaternionf other)
    cpdef Quaternionf inverse(Quaternionf self)
    cpdef Quaternionf normalize(Quaternionf self)
    cpdef Quaternionf slerp(Quaternionf self, Quaternionf other, float blend)
    cpdef Quaternionf concatenate(Quaternionf self, Quaternionf other)
    cpdef Quaternionf negate(Quaternionf self)
    cpdef Vector4f to_axis_angle_rotation(Quaternionf self)
    cpdef Matrix4f to_rotation_matrix(Quaternionf self)
    cpdef Vector3f to_yaw_pitch_roll(Quaternionf self)
    cpdef _cp_richcmp(self, Quaternionf other, int op)
    #TODO:Add method to bind to uniform