# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016, Brian Thorpe. All Rights Reserved.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
from libc.math cimport sin, cos, sqrt, acos, abs, atan2, M_PI
from cymath.vector3 cimport Vector3
from cymath.matrix4 cimport Matrix4, _create_scale, _create_from_quaternion, _create_translation
from cymath.quaternion cimport Quaternion

cdef class Transform:
    cdef Vector3 _local_position
    cdef Vector3 _local_scaling
    cdef Quaternion _local_rotation
    cdef Vector3 _global_position
    cdef Vector3 _global_scaling
    cdef Quaternion _global_rotation
    cdef Matrix4 _local_matrix
    cdef Matrix4 _global_matrix
    cdef Transform _parent
    cdef list _children
    cdef void _update_local_matrix(Transform self)
    cdef void _update_global(Transform self)
    cpdef set_parent(self, Transform parent, bint use_global=?)
    cdef _attach(self, Transform parent)
    cdef _detatch(self)
    cpdef set_global_state(self, Vector3 global_position, 
                           Vector3 global_scaling, Quaternion global_rotation)
    cpdef set_global_position(self, Vector3 global_position)
    cpdef set_global_scaling(self, Vector3 global_scaling)
    cpdef set_global_rotation(self, Quaternion global_rotation)
