# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016, Brian Thorpe. All Rights Reserved.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from libc.math cimport sqrt, acos, atan2
from cymath.math_structs cimport double3

from cymath.matrix4 cimport Matrix4
from cymath.vector2 cimport Vector2
from cymath.vector4 cimport Vector4
from cymath.vector3f cimport Vector3f

cdef class Vector3:
    cdef double3 _vector
    cpdef double magnitude_squared(Vector3 self)    
    cpdef double magnitude(Vector3 self)
    cpdef double dot(Vector3 self, Vector3 other)
    cpdef Vector3 cross(Vector3 self, Vector3 other)
    cpdef Vector3 add(Vector3 self, Vector3 other)    
    cpdef Vector3 subtract(Vector3 self, Vector3 other)
    cpdef Vector3 multiply(Vector3 self, double scalar)
    cpdef Vector3 divide(Vector3 self, double scalar)
    cpdef Vector3 normalize(Vector3 self)
    cpdef Vector3 negate(Vector3 self)
    cpdef double angle_between(Vector3 self, Vector3 other)
    cpdef double signed_angle_between(Vector3 self, Vector3 other, Vector3 normal)
    cpdef Vector3 transform(Vector3 self, Matrix4 matrix)
    cpdef Vector3f to_float(Vector3 self)
    cpdef _cp_richcmp(Vector3 self, Vector3 other, int op)

    cdef double* to_ptr(Vector3 self)
    
    #TODO:
    cpdef Vector3 catmullrom(Vector3 value1, Vector3 value2, Vector3 value3, Vector3 value4, double amount)
    cpdef double distance(Vector3 vector1, Vector3 vector2)
    cpdef double distance_squared(Vector3 value1, Vector3 value2)
    cpdef Vector3 hermite(Vector3 value1, Vector3 tangent1, Vector3 value2, Vector3 tangent2, double amount)
    cpdef Vector3 smooth_step(Vector3 value1, Vector3 value2, double amount)
    cpdef Vector3 reflect(Vector3 vector, Vector3 normal)
    cpdef Vector3 clamp(Vector3 self, Vector3 vector_min, Vector3 vector_max)
    cpdef Vector3 min(Vector3 self, Vector3 other)
    cpdef Vector3 max(Vector3 self, Vector3 other)